;; - nodes.lsp v. 1.1
;;
;; These routines can be used to set up a cluster of newLISP processes
;; to operate in parallel.  When running this on OpenMosix
;; processes will be redistributed automatically to other CPUs.
;;
;;

;; Start a newLISP process and return a socket handle to talk to.
;; The returned handle/socket can be used in subsequent
;; (Node:xxx ...) calls.
;;
;; example:
;;	(Node:start 1001)		; start Node listening on 1001
;;      (Node:eval-str 1001 "(+ 3 4)")  ; send task to evaluate
;;      (if (Node:ready? 1001)		; check if done 
;;		(Node:read 1001) 	; => "7"
;;	(Node:stop 1001)		; close node 1001
;;
;; See the function 'test-nodes' at the bottom of the file for a more
;; comprehensive example involving several nodes.
;;

(context 'Node)

(set 'sessions '()) ; contains port socket pairs


;; start a new node on a specific commmunications port
;; optionally specify a startup file
;;

(define (start port init)
	(if (process (string "newlisp -c -p " port " " (or init "")))
		(begin
		  	(set 'cnt 0)
			(set 'sock nil)
		  	(while (and (not sock) (< cnt 10))
				(set 'sock (net-connect "localhost" port))
				(sleep 500)
				(inc 'cnt))
			(if sock (push (list port sock) sessions)))))
			
;; Close a node process or all processes.
;;
;; If a port is given close that process,
;; if none is given close them all.
;;

(define (stop port)
	(if port 
		(begin
			(net-close (lookup port sessions))
			(replace (assoc port sessions) sessions) true)
		(begin
			(dolist (s sessions) (net-close (last s)))
			(set 'sessions '()) true )))


;; Check if a node process is ready for reading
;; a result - return 'true' or 'nil'.
;;

(define (ready? port)
	(set 'sock (lookup port sessions))
	(if sock (net-select (lookup port sessions) "r" 100)))


;; Evaluate a string on a node newlisp process
;; note, that node processes maintain state
;; until closed with (Node:stop port).
;;
;; Returns the number of characters sent.

(define (eval-str port str)
	(set 'sock (lookup port sessions))
	(if sock (net-send sock (append str "\n"))))


;; Read the result string from the node process
;;
(define (read port)
	(set 'sock (lookup port sessions))
	(if sock (begin
		(net-receive sock 'buff (net-peek sock))
		buff)))


(context 'MAIN)


;; test functions in Nodes
;;


(define (test-nodes)

	;; create a program file
	(write-file "ntest.lsp"
[text]
(define (long-task x)
	(sleep 5000)
	x)
[/text]
	)

	;; start the nodes
	(dolist (n '(2001 2002 2003))
		(println "starting node: " (Node:start n "ntest.lsp")))
		
	(println "starting tasks at: " (date (apply date-value (now))))

	(Node:eval-str 2001 "(long-task 1)")
	(Node:eval-str 2002 "(long-task 2)")
	(Node:eval-str 2003 "(long-task 3)")

	(println "wait 5 seconds ...")
	(while Node:sessions
		(dolist (s Node:sessions)
			(set 'p (first s))
			(if (Node:ready? p)
				(begin  
					(println "result node " p ":" (Node:read p))
					(Node:stop p)))))

	(println "finishing at: " (date (apply date-value (now))))

	;; delete temporary program file
	(delete-file "ntest.lsp")
)


;; eof ;;

		

