;; sqlite.lsp - module for SQLite newLISP bindings
;;
;; version 1.0 - initial release,  tested on WinXP/freeBSD and LINUX
;; version 1.1 - 'select' statment now detected by num-cols > 0 in function 'sql'
;; version 1.2 - now check for null-pointer to resuil on math exceptions
;; version 1.3 - crash protection for multiple close  end error message
;; version 1.4 - 'sql' returned error message on errors instead of 'nil'
;; version 1.5 -  new function (SQLite:tables) and (SQLite:columns tableName)
;; version 1.6 - changes name of 'array' which is now reserved symbol snce 7.5.1
;; version 1.7 - change for version detection for 8.1.0
;; version 1.8 - 'get-integer' to 'get-int'
;;
;; required: sqlite.dll or libsqlite.so from http://www.sqlite.org
;;
;;
;; USAGE:
;;
;; (SQLite:open "MYDB")				; opens/creates a database returns a handle (ignore)
;;						; or 'nil' on failure
;;
;; (SQLite:sql "select * from mytable;")	; makes a sql query, returns result
;;
;; (SQLite:changes)				; returns number of rows affected
;;
;; (SQLite:rowid)				; returns last row id from insert
;;
;; (SQLite:tables)				; returns a list of all tables in a database
;;
;; (SQLite:columns tableName)			; returns a list of column lables for tableName
;;
;; (SQLite:close)				; closes the database
;;
;; (SQLite:error)                               ; returns error text
;;
;; (SQLite:timeout 10000)			; sets bust timeout to 10,000 ms, returns db handle (ignore)

(context 'SQLite)

;; set libray to path-name of the library on your platform OS
;;
(if (< (& 0xF (last (sys-info))) 5)
	(set 'library "/usr/local/lib/libsqlite.so") ;; LINUX and other UNIXs
	(set 'library "/sqlite/sqlite.dll"))	     ;; CYGWIN, WIn32

(import library "sqlite_open" "cdecl")
(import library "sqlite_close" "cdecl")
(import library "sqlite_exec" "cdecl")
(import library "sqlite_compile" "cdecl")
(import library "sqlite_step" "cdecl")
(import library "sqlite_finalize" "cdecl")
(import library "sqlite_last_insert_rowid" "cdecl")
(import library "sqlite_changes" "cdecl")
(import library "sqlite_busy_timeout" "cdecl")


(set 'db nil)
(set 'error-message "")


;; open or ceate a database ;;;
;; if the database does exist it gets opened, else a new database
;; with the name given is created.
;;
;;
(define (SQLite:open db-name)
	(set 'result (sqlite_open db-name 0 0))
	(if (= result 0)
		(set 'db nil)
		(set 'db result)))


;; close the currently open database ;;;
;;
;;
(define (SQLite:close) 		;;  overwrite the close in MAIN
	(if db (begin
		(sqlite_close db)
		(set 'db nil)
		true)))

;; execute an SQL statement ;;;
;;
;; For 'select' statements an array of the results is returned or '()
;; for the empty set. For other statements 'true is returned for a
;; successful outcome. On failure 'nil is returened (SQLite:error)
;; can be used to retrive the error text.
;;
;;
(define (sql sql-str)
	(set 'result nil 'done nil 'error-message "")
	(set 'sqarray '());
	(set 'col-names '());

	;; set up parameters for sqlite_compile() call
	(set 'ppvm "\000\000\000\000") 
	(set 'pperr "\000\000\000\000")

	;; compile the sql statment
	(if db (set 'result (sqlite_compile db sql-str 0 ppvm pperr)))

	;; set up parameters for sqlite_step() call
	(set 'pvm (get-int ppvm))
	(set 'pn "\000\000\000\000")
	(set 'pvalue "\000\000\000\000")
	(set 'pcname "\000\000\000\000")

	;; execute the compiled statement
	(if (= result 0) 
		(while (not done) 
			;; execute statement until done/101 or 
			(set 'result (sqlite_step pvm pn pvalue pcname))
			(set 'num-cols (get-int pn))
			(if (empty? col-names) (set 'col-names (get-names pcname num-cols)))
			(if (= result 100)
				(push (get-values pvalue num-cols) sqarray)
				(set 'done true) ;; received done/101 or error
			))
		
                )
	;; if done/101 finalize
	(if (= result 101) 
		(begin
			(set 'result (sqlite_finalize pvm 0))
			;; for 'select' statements return the array else 'true'
			(if (> num-cols 0) (reverse sqarray) true))
		(if (= result 0) true (set-error pperr))))


(define (get-values values cols) 
	(set 'row '())
	(set 'values (get-int values))
	(dotimes (idx cols)
		(set 'ps (get-int (+ values (* idx 4))))
	;;	(println "ps: " ps)
		(if (= ps 0)	       ;; check for null pointer to result
			(push "" row)
			(push (get-string ps) row)))
	(reverse row))


(define (get-names names cols) 
	(set 'row '())
	(set 'names (get-int names))
	(dotimes (idx cols)
		(set 'ps (get-int (+ names (* idx 4))))
	;;	(println "ps: " ps)
		(if (= ps 0)	       ;; check for null pointer to result
			(push "" row)
			(push (get-string ps) row)))
	(reverse row))



;; return last row id from last 'insert'
;;
(define (rowid)
	(if db (sqlite_last_insert_rowid db)))

;; return a list of tables in database
;;
(define (tables)
	(if db (begin
		(set 'lst (sql "select tbl_name from sqlite_master")) ))
		(if lst (set 'lst (first (transpose lst)))))



;; return a list of column names for a table
;;
(define (columns aTable)
        (if (list? (sql (append "select * from " aTable " where 0;")))
                col-names))


;; return number of rows changed/affected
;;
(define (changes)
	(if db (sqlite_changes db)))



;; sets busy tiomeout in milliseconds
;;
(define (timeout ms)
	(if db (sqlite_busy_timeout db ms)))



;; return the error text of the last error occured in SQLite:sql
;;
;;
(define (error) error-message)
	
(define (set-error pperror)
	(set 'perror (get-int pperror))
	(if (!= 0 perror)
		(set 'error-message (get-string  perror)))
	nil)


(context 'MAIN)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; test the database routines
;;
;;
(define (test-sqlite)
	(if (SQLite:open "TEST") 
		(println "database opened/created,  ... Ok")
		(println "problem opening/creating database"))

	(if (SQLite:sql "create table fruits (name CHAR(20), qty INT(3));")
		(println "created table fruits,  ... Ok")
		(println "problem creating table fruits"))

	(if (SQLite:sql "insert into fruits values ('apples', 11);")
		(println "inserted, last row id: " (SQLite:rowid) ",  ... Ok")
		(println "problem inserting row"))

	(if (SQLite:sql "insert into fruits values ('oranges', 22);")
		(println "inserted, last row id: " (SQLite:rowid) ",  ... Ok")
		(println "problem inserting row"))

	(if (SQLite:sql "insert into fruits values ('bananas', 33);")
		(println "inserted, last row id: " (SQLite:rowid) ",  ... Ok")
		(println "problem inserting row"))

	(set 'sqarray (SQLite:sql "select * from fruits;"))

	(if sqarray
		(println "selected rows: " sqarray ",  ... Ok")
		(println "problem with select"))

	(if (SQLite:sql "delete from fruits where 1;")
		(println "deleted, rows affected: " (SQLite:changes) ",  ... Ok")
		(println "problem deleting rows"))

	(if (list? (SQLite:tables) )
		(println "tables: " (SQLite:tables) ", ... Ok")
		(println "problem in SQLite:tables"))

	(if (list? (SQLite:columns "fruits") )
		(println "columns: " (SQLite:columns "fruits") ", ... Ok")
		(println "problem in SQLite:columns"))


	(if (SQLite:sql "drop table fruits;")
		(println "table fruits dropped,  ... Ok")
		(println "problem dropping table fruits"))

	

	(SQLite:close))


;; eof ;;
