;
; txt2pdf.lsp - convert text to pdf file format
;
; version 1.0 - 2004-02-14 - released as a newLISP module
;
; below the copyright notice of the original 'C' program:
;
;    Copyright 1998
;    P. G. Womack, Diss, Norfolk, UK.
;    "BugBear"
;    Do what you like, but don't claim you wrote it.
;
; converted by Nigel to newLISP 2004-02-05
;
; added interface functions for file name or string buffer L.M. 2004-02-06
;
; usage:
;
;	(TXT2PDF:file inFileName outFileName formFeedLines)
;
; or:
;
;	(TXT2PDF:text textBuffer outFileName formFeedLines)
;
; The 'formFeedLines' parameter is optionally the number of lines
; before a page feed is inserted. The page feed can also be given
; as a formfeed character ASCII 12 on a line by itself inside the text
; then the parameter can be ommitted
;
; example:
;
;	(TXT2PDF:file "myFile.txt" "myFile.pdf" 75)
;
; converts the file 'myFile.txt' to 'myFile.pdf' inserting a page feed
; every 72 lines
;
;    (TXT2PDF:text "this is a test page 1\n\012\nThis on page 2\n" "demo.pdf")
;
;
; other options:
;
;	(TXT2PDF:file inFileName outFileName formFeedLines footerLines prePageNr postPageNr)
;
; example:
;
;    (TXT2PDF:file "myFile.txt" "myFile.pdf" 75 3 "myFile.txt -" "-")
;
; This will include a footer of 3 lines, the first 2 lines ar empty on the
; 3rd line it will print "myFile.txt -#-" where # is the page number
;
; NOTE, of footerLines are specified, the text should not contain any form feed characters!
;
;


(context 'TXT2PDF)

(setq page_width 594.0 )
(setq page_depth 828.0 )
(setq margin 30.0 )
(setq font_size 10.0 )
(setq lead_size 10.0 )
(setq object_id 1 )

(setq num_pages 0)
(setq pages '())

(define (store_page id) 
	(push id pages -1) (inc 'num_pages))

(setq num_xrefs 0)
(setq xrefs '()) ; holdsposition of xrefs as assoc list

(define (start_object id) 
	(push (list id (seek pdfout)) xrefs)
	(write-buffer pdfout (format "%d 0 obj\n" id)))

(setq stream_id 0)
(setq stream_len_id 0)
(setq ypos 0.0)

(define (start_page) 
	(setq stream_id object_id)
	(inc 'object_id)
	(setq stream_len_id object_id)
	(inc 'object_id)
	(start_object stream_id)
	(write-buffer pdfout (format "<< /Length %d 0 R >>\n" stream_len_id))
	(write-buffer pdfout "stream\n")
	(setq stream_start (seek pdfout))
	(write-buffer pdfout (format "BT\n/F0 %g Tf\n" font_size))
	(setq ypos (sub page_depth margin))
	(write-buffer pdfout (format "%g %g Td\n" margin ypos))
	(write-buffer pdfout (format "%g TL\n" lead_size)))

(define (end_page) 
	(setq page_id object_id)
	(inc 'object_id)
	(store_page page_id)
	(write-buffer pdfout "ET\n")
	(setq stream_len (- (seek pdfout) stream_start))
	(write-buffer pdfout "endstream\nendobj\n")
	(start_object stream_len_id)
	(write-buffer pdfout 
		(format "%d\nendobj\n" stream_len)) ; "%ld\nendobj\n" newlisp int is 32 bit
	(start_object page_id)
	(write-buffer 
		pdfout (format "<</Type/Page/Parent %d 0 R/Contents %d 0 R>>\nendobj\n" page_tree_id stream_id)))

(define (do_text pdfgets)
	(start_page)
	(while (setq buffer (pdfgets))
		(if (< ypos margin)
			(begin
				(end_page)
				(start_page)))
		(if (= (length buffer) 0)
		(write-buffer pdfout "T*\n")
		(if (= (first buffer) "\012"); formfeed code
			(begin
				(end_page)
				(start_page))
			(begin
				(write-buffer pdfout "(")
				(setq s -1)
				(if (!= (nth -1 buffer) "\000") 
					(setq buffer (append buffer "\000"))) ; protect from endless last char
				(while (!= (setq c (nth (inc 's) buffer)) "")
					(case c
						("(" (write-buffer pdfout "\\"))
						(")" (write-buffer pdfout "\\"))
						("\\" (write-buffer pdfout "\\")))
						(write-buffer pdfout c))

				(write-buffer pdfout ")'\n"))))

		(setq ypos (sub ypos lead_size)))
		(end_page))


(define (txt2pdf pdfgets pdffilename)
	(set 'pdfout (open pdffilename "write")) ; mod to have txt2pdf to return nil if open fails
	(if (= pdfout nil) nil 
		(begin ; do else if file opened ok
			(write-buffer pdfout "%%PDF-1.0\n")
			(setq page_tree_id object_id)
			(inc 'object_id)
			(do_text pdfgets)
			(setq font_id object_id)
			(inc 'object_id)
			(start_object font_id)
			(write-buffer pdfout 
				"<</Type/Font/Subtype/Type1/BaseFont/Courier/Encoding/WinAnsiEncoding>>\nendobj\n")
			(start_object page_tree_id)
			(write-buffer pdfout (format "<</Type /Pages /Count %d\n" num_pages))
			(write-buffer pdfout "/Kids[\n")
			(dolist (ppid pages) 
				(write-buffer pdfout (format "%d 0 R\n" ppid)))
			(write-buffer pdfout "]\n")
			(write-buffer pdfout 
				(format "/Resources<</ProcSet[/PDF/Text]/Font<</F0 %d 0 R>> >>\n" font_id))
			(write-buffer pdfout (format "/MediaBox [ 0 0 %g %g ]\n" page_width page_depth))
			(write-buffer pdfout ">>\nendobj\n")
			(setq catalog_id object_id)
			(inc 'object_id)
			(start_object catalog_id)
			(write-buffer pdfout (format "<</Type/Catalog/Pages %d 0 R>>\nendobj\n" page_tree_id))
			(setq start_xref (seek pdfout))
			(write-buffer pdfout "xref\n")
			(write-buffer pdfout (format "0 %d\n" object_id))
			(write-buffer pdfout "0000000000 65535 f \n")
			(dotimes (i (- object_id 1)) 
				(write-buffer pdfout 
					(format "%010d 00000 n \n" (lookup (+ i 1) xrefs)))) ;"%010ld 00000 n \n"
			(write-buffer pdfout (format "trailer\n<<\n/Size %d\n/Root %d 0 R\n>>\n" object_id catalog_id))
			(write-buffer pdfout (format "startxref\n%d\n%%%%EOF\n" start_xref));"startxref\n%ld\n%%%%EOF\n"
			(close pdfout))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; added LM 2004-02-06 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(define (file inFileName outFileName formFeedLines footerLines prePageNr postPageNr)
	(set 'countLines 0)
	(set 'countPages 0)
	(set 'outOfLines nil)
	(get-params)
	(set 'input-lines (parse (read-file inFileName) "\n|\r\n" 0))
	(txt2pdf get-lines outFileName))

(define (text inBuffer outFileName formFeedLines footerLines prePageNr postPageNr)
	(set 'countLines 0)
	(set 'countPages 0)
	(set 'outOfLines nil)
	(get-params)
	(set 'input-lines (parse inBuffer "\n|\r\n" 0))
	(txt2pdf get-lines outFileName))

(define (get-params)
	(if (not formFeedLines) (set 'formFeedLines 75))
	(if (not prePageNr) (set 'prePageNr " "))
	(if (not postPageNr) (set 'postPageNr " ")))

(define (get-lines) ;; called from (txt2pdf ...)
	(cond
		( (= countLines formFeedLines)
			(set 'countLines 0)
			(if outOfLines nil "\012"))

		( (and footerLines (= countLines (- formFeedLines 1)))
			(inc 'countLines)
			(inc 'countPages)
			(string prePageNr countPages postPageNr))

		( (and footerLines (>= countLines (- formFeedLines footerLines)))
			(inc 'countLines)
			" ")	

		( true
			(inc 'countLines)
			(if (set 'popLine (pop input-lines))
				popLine
				(begin (set 'outOfLines true) " ")))))
			

(context 'MAIN)


;; eof ;;
